package tab.list

import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material.icons.filled.*
import androidx.compose.material3.*
import androidx.compose.runtime.*
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.input.nestedscroll.nestedScroll
import androidx.compose.ui.semantics.semantics
import androidx.compose.ui.semantics.traversalIndex
import androidx.compose.ui.unit.dp
import cafe.adriel.voyager.core.model.rememberNavigatorScreenModel
import cafe.adriel.voyager.navigator.LocalNavigator
import cafe.adriel.voyager.navigator.currentOrThrow
import cafe.adriel.voyager.navigator.tab.Tab
import cafe.adriel.voyager.navigator.tab.TabOptions
import component.TopBar
import tab.LoadingScreen

object ListMain : Tab {
    override val options: TabOptions
        @Composable
        get() {
            val title = "E Numbers List"

            return remember {
                TabOptions(
                    index = 0u,
                    title = title
                )
            }
        }

    @OptIn(ExperimentalMaterial3Api::class)
    @Composable
    override fun Content() {
        val navigator = LocalNavigator.currentOrThrow
        val screenModel = navigator.rememberNavigatorScreenModel { ListScreenModel() }
        val state by screenModel.state.collectAsState()

        val searchActive = rememberSaveable { mutableStateOf(false) }

        val scrollBehavior = TopAppBarDefaults.pinnedScrollBehavior(rememberTopAppBarState())

        Scaffold(
            modifier = Modifier.nestedScroll(scrollBehavior.nestedScrollConnection),
            topBar = {
                if (!searchActive.value) {
                    TopBar(scrollBehavior = scrollBehavior, searchActive = searchActive)
                } else {
                    Search(searchActive = searchActive)
                }
            }
        ) { innerPadding ->
            Column(
                modifier = Modifier.padding(innerPadding)
            ) {
                when (val s = state) {
                    is ListScreenModel.State.Loading -> LoadingScreen("E Numbers").Content()
                    is ListScreenModel.State.Result -> ENumberList(s.eNumbers).Content()
                }
            }
        }

        LaunchedEffect(currentCompositeKeyHash) {
            screenModel.getENumbers()
        }
    }

    @OptIn(ExperimentalMaterial3Api::class)
    @Composable
    private fun Search(searchActive: MutableState<Boolean>) {
        var text by rememberSaveable { mutableStateOf("") }

        SearchBar(
            modifier = Modifier
                .semantics { traversalIndex = 0f },
            query = text,
            onQueryChange = { text = it },
            onSearch = { searchActive.value = false },
            onActiveChange = { searchActive.value = it },
            active = searchActive.value,
            leadingIcon = {
                IconButton(onClick = { searchActive.value = false }) {
                    Icon(Icons.AutoMirrored.Filled.ArrowBack, "Exit Search")
                }
            },
            trailingIcon = {
                IconButton(onClick = { text = "" }) {
                    Icon(imageVector = Icons.Filled.Clear, contentDescription = "Clear Search")
                }
            }
        ) {
            Column(Modifier.verticalScroll(rememberScrollState())) {
                repeat(4) { idx ->
                    val resultText = "Suggestion $idx"
                    ListItem(
                        headlineContent = { Text(resultText) },
                        supportingContent = { Text("Additional info") },
                        leadingContent = { Icon(Icons.Filled.Star, contentDescription = null) },
                        colors = ListItemDefaults.colors(containerColor = Color.Transparent),
                        modifier = Modifier
                            .clickable {
                                text = resultText
                                searchActive.value = false
                            }
                            .fillMaxWidth()
                            .padding(horizontal = 16.dp, vertical = 4.dp)
                    )
                }
            }
        }
    }
}
